/**
 * 应用卡片滑动模块
 * @author linwang6
 */
function appSwipe(configParam){

    // 参数合法性校验
    if (!configParam || typeof configParam != 'object') {
        console.log('Error -> configParam must be an object');
        return;
    }
    if (!configParam.swipe_frame || !configParam.swipe_area) {
        console.log('Error -> swipe_frame and swipe_area must be DOM elements');
        return;
    }
    
    var swipe_frame = configParam.swipe_frame;
    var swipe_area = configParam.swipe_area;
    console.log("appSwipe get configParam finished");
    // 事件监听初始化
    initEvent();
    
    function initEvent(){
        console.log("appSwipe initEvent");
        var begin, moved; // bool类型
        var startPosTemp, startPos, stopPos; // startPosTemp为每次滑动过程中选取的点，startPos为初始选择的参考点
        var distChanged = []; // 数组，记录每次onMove过程中位移变化量
        var startX, startY;
        var moveThreshold = 0.9; // 最后一次onMove执行时的位移参考阈值
        var appItemNum = 0; // 推荐应用数目
        var switchIndex = 0; // 滑动标记
        
        var CurLeftPos = 0; // 滑动区域（最左端）相对于框架的当前位置
        var CurLeftPosTemp = 0; // 临时量，用于控制滑动区域随手指运动
        var moveTimes; // onMove执行的次数
        var isVerticalMove; // bool类型，是否为竖直滑动
        
        swipe_area.ontouchstart = onStart;
        swipe_area.ontouchmove = onMove;
        swipe_area.ontouchend = onEnd;
        swipe_area.ontouchcancel = onCancel;
        
        reset();
        
        // 获取滑动区域相关宽度值
        var nodes = null;
        var swipe_frame_width = 0; // 滑动框架宽度，等于屏幕分辨率宽
        var swipe_area_width = 0; // 有效滑动区域宽度
        var appItemWidth = 0; // 当个应用区域宽度
        var spaceWidth = 0; // 滑动区域首尾的间距
        var marginWidth = 0; // 相邻应用之间的间距
        var borderShift = 0; // 切换时滑动区域的滑动距离（处于端点时）
        var middleShift = 0; // 切换时滑动区域的滑动距离（处于中间时）
        var recursionTimes = 0; // 递归迭代次数
        
        function getWidth(){
            nodes = swipe_area.childNodes;
            recursionTimes++;
            if (recursionTimes > 10) {
                return;
            }
            // 至少有2个应用才执行滑动
            if (nodes && nodes.length >= 5) {
                appItemNum = (nodes.length - 1) / 2;
                swipe_frame_width = swipe_frame.clientWidth;
                swipe_area_width = swipe_area.clientWidth;
                appItemWidth = swipe_area.childNodes[1].clientWidth;
                spaceWidth = swipe_area.childNodes[0].clientWidth;
                marginWidth = swipe_area.childNodes[2].clientWidth;
                if (swipe_frame_width != 0 && swipe_area_width != 0 && appItemWidth != 0 && spaceWidth != 0 && marginWidth != 0) {
                    borderShift = spaceWidth + marginWidth + (3 * appItemWidth - swipe_frame_width) / 2;
                    middleShift = appItemWidth + marginWidth;
                    console.log("appSwipe onStart swipe_frame_width: " + swipe_frame_width);
                    console.log("appSwipe onStart swipe_area_width: " + swipe_area_width);
                    console.log("appSwipe onStart appItemWidth: " + appItemWidth);
                    console.log("appSwipe onStart spaceWidth: " + spaceWidth);
                    console.log("appSwipe onStart marginWidth: " + marginWidth);
                    console.log("appSwipe onStart borderShift: " + borderShift);
                    console.log("appSwipe onStart middleShift: " + middleShift);
                    return;
                }
            }
            setTimeout(function(){
                getWidth();
            }, 200);
        }
        
        function onStart(e){
            reset();
			swipe_area.style.webkitTransition = "left 0.0s"; // onStart时去动画效果
            console.log("appSwipe onStart CurLeftPos: " + CurLeftPos);
            getWidth();
            if (swipe_frame_width == 0 || swipe_area_width == 0 || appItemWidth == 0 || spaceWidth == 0 || marginWidth == 0) {
                return;
            }
            console.log("appSwipe begin: " + begin);
            if (begin) {
                return;
            }
            if (isNaN(CurLeftPos)) {
                console.log("appSwipe onStart NaN CurLeftPos!");
                CurLeftPos = 0;
				CurLeftPosTemp = CurLeftPos;
            }
            
            event.stopPropagation();
            begin = true;
            startX = getCoord(e, 'X');
            startPosTemp = startX;
            startPos = startX;
            startY = getCoord(e, 'Y');
            console.log("appSwipe onStart startPos: " + startPos);
        }
        
        function onCancel(e){
            console.log("appSwipe onCancel");
            isVerticalMove = false;
			e.stopPropagation();
            moved = true; // 针对点击动作又取消的情况
            moveEnd();
        }
        
        function onMove(e){
            moveTimes++;
            if (startX && startY) {
                var x = getCoord(e, 'X');
                var y = getCoord(e, 'Y');
                var deltaX = x - startX;
                var deltaY = y - startY;
                
                // 如果判定是纵向移动，不作处理
                if (Math.abs(deltaX) < Math.abs(deltaY) * 2) {
                    moved = true;
                    if (moveTimes <= 2) {
                        isVerticalMove = true;
                        console.log("appSwipe onMove isVerticalMove: " + isVerticalMove);
                    }
                }
                
                if (!isVerticalMove) {
                    console.log("appSwipe onMove isVerticalMove: " + isVerticalMove);
                    console.log("appSwipe onMove moveTimes: " + moveTimes);
                    
                    e.preventDefault();
                    e.stopPropagation();
                    stopPos = x;
					var distChangedTemp = 1.0 * (stopPos - startPosTemp);
                    distChanged[moveTimes - 1] =  distChangedTemp // moveTimes从1开始，数组下标从0开始
                    console.log("appSwipe onMove distChanged: " + distChangedTemp);
                    
                    CurLeftPosTemp += distChangedTemp;
                    console.log("appSwipe onMove CurLeftPosTemp: " + CurLeftPosTemp);
                    swipe_area.style.left = CurLeftPosTemp + 'px';
                    startPosTemp = stopPos;
					
                    moved = true;
                    stopPos = getCoord(e, 'X');
                    console.log("appSwipe onMove CurLeftPos: " + CurLeftPos);
                }
            }
        }
        
        function onEnd(e){
            e.stopPropagation();
            moveEnd();
            reset();
        }
        
        function moveEnd(){
            console.log("appSwipe onEnd moved: " + moved);
            if (!moved) {
                return;
            }
            
            if (!isVerticalMove) {
                if (moveTimes >= 2) {
                    // 向右有效滑动
                    if (distChanged[moveTimes - 1] > moveThreshold || distChanged[moveTimes - 2] > moveThreshold) {
                        if (switchIndex == -1 || switchIndex == -1 * (appItemNum - 1)) {
                            CurLeftPos += borderShift;
                            switchIndex++;
                            exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.app, "swipeAppPage"]);
                        }
                        else 
                            if (switchIndex < -1 && switchIndex > -1 * (appItemNum - 1)) {
                                CurLeftPos += middleShift;
                                switchIndex++;
                                exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.app, "swipeAppPage"]);
                            }
                        
                        console.log("appSwipe onEnd CurLeftPos: " + CurLeftPos);
                        console.log("appSwipe onEnd switchIndex: " + switchIndex);
                    }
                    
                    // 向左有效滑动
                    if (distChanged[moveTimes - 1] < -1 * moveThreshold || distChanged[moveTimes - 2] < -1 * moveThreshold) {
                        if (switchIndex == 0 || switchIndex == -1 * (appItemNum - 2)) {
                            CurLeftPos -= borderShift;
                            switchIndex--;
                            exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.app, "swipeAppPage"]);
                        }
                        else 
                            if (switchIndex < 0 && switchIndex > -1 * (appItemNum - 2)) {
                                CurLeftPos -= middleShift;
                                switchIndex--;
                                exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.app, "swipeAppPage"]);
                            }
                        
                        console.log("appSwipe onEnd CurLeftPos: " + CurLeftPos);
                        console.log("appSwipe onEnd switchIndex: " + switchIndex);
                    }
                }
                
                CurLeftPosTemp = CurLeftPos;
                swipe_area.style.webkitTransition = "left 0.3s"; // 动画平滑效果
                swipe_area.style.left = (CurLeftPos - 1) + "px";
            }
        }
        
        function reset(){
            moved = false;
            begin = false;
            isVerticalMove = false;
            moveTimes = 0;
            distChanged = [];
        }
        
        function getCoord(e, c){
            var org = e.originalEvent, ct = e.changedTouches;
            return ct || (org && org.changedTouches) ? (org ? org.changedTouches[0]['page' + c] : ct[0]['page' + c]) : e['page' + c];
        }
    }  
}
