define(function(){
    var res_path = "./app/res/";
    
    var baseInfo = null;
    var clientVersion = null; //客户端版本号
    var allPackageNames = null;//保存所有应用程序的包名
    var installedAppIndex = [];//已安装的应用程序下标
    var uninstalledAppIndex = [];//未安装的应用程序下标
    var canSwitchAppsIndex = null;//当前可以在卡片进行切换展示的20个应用下标
    var currentShowAppsIndex = [];//当前正在显示的应用下标
    var PAGE_SHOW_NUM = 4;//一屏显示的应用数目
    var MAX_PAGE_NUM = 5;//换一换最多屏数
    var SWIPE_SHOW_NUM = 4; // 当前展示的可供滑动的应用数目
    var appCardData = null; //应用卡片数据
    var appCardDiv = null;
    var isAppCardCreate = false; //标识应用卡片视图是否已经创建
    var isAppCardRequested = false; //标识应用卡片是否请求过
    var isAppCardRequesting = false; //标识应用卡片是否正在请求
    var requestCondition = 0;//0:无网络，不需要请求;1:不判断网络，直接请求;2：判断网络，再请求
    var VERSION_THRESHOLD = 1909; //老版本的阈值
    
    var zdOffDom = null;
    var speechCaseDom = null;
    var appCard = null;
    
    /**安装按钮的不同状态的文本*/
    var installButtonText = {
        "install": "安装",
        "open": "打开"
    };
    
    var cpaCodeKey = "appsearch";
    var opCodeKey = { //记录操作日志的key
        "item": "item",
        "category": "category",
        "switchBtn": "switchBtn",
        "moreBtn": "moreBtn"
    };
    
    function checkString(s){
        if (s && s.length > 0) {
            return true;
        }
        return false;
    }
    
    function filterRequestResult(requestResult){
        console.log("filterRequestResult(), requestResult is " + requestResult);
        if (!requestResult || !requestResult.content) {
            return null;
        }
        var content = requestResult.content;
        if (!content.apps || content.apps.length < SWIPE_SHOW_NUM || !content.bottomButton || content.bottomButton.length < 1) {
            return null;
        }
        
        try {
            var filterResult = {};
            filterResult.apps = [];
			// 增加count和bimgUrl字段过滤
            for (var i = 0; i < content.apps.length; i++) {
                var app = content.apps[i];
                if (app && checkString(app.id) && checkString(app.packageName) && checkString(app.autoOpen) &&
                checkString(app.name) &&
                checkString(app.imgUrl) &&
                checkString(app.desc) &&
                checkString(app.downUrl) &&
				checkString(app.bimgUrl) &&
				checkString(app.count)) {
                    filterResult.apps.push(app);
                }
            }
            if (filterResult.apps.length < SWIPE_SHOW_NUM) {
                return null;
            }
            
//            filterResult.category = [];
//            for (var i = 0; i < content.category.length; i++) {
//                var category = content.category[i];
//                if (category && checkString(category.name) && checkString(category.url)) {
//                    filterResult.category.push(category);
//                }
//            }
//            if (filterResult.category.length < 3) {
//                return null;
//            }
            
            filterResult.bottomButton = [];
            for (var i = 0; i < content.bottomButton.length; i++) {
                var bottomButton = content.bottomButton[i];
                if (bottomButton && checkString(bottomButton.name) && checkString(bottomButton.url)) {
                    filterResult.bottomButton.push(bottomButton);
                }
            }
            if (filterResult.bottomButton.length < 1) {
                return null;
            }
            
            return filterResult;
        } 
        catch (err) {
            return null;
        }
    }
    
    function handleRequestSuccess(requestResult){
        console.log("handleRequestSuccess, requestResult is " + requestResult);
        
        isAppCardRequesting = false;
        
        /**
         * 根据返回不同的状态进行不同处理
         */
        try {
            var requestResultJson = JSON.parse(requestResult);
            var status = requestResultJson.status;
            var errorCode = requestResultJson.errorCode;
            console.log("status is " + status + ", errorCode is " + errorCode);
            if ("000000" == errorCode) { //数据成功返回
                //解析返回结果
                var filterResult = filterRequestResult(requestResultJson);
                console.log("filterResult is " + JSON.stringify(filterResult));
                if (filterResult) {
                    //回调给框架请求成功返回
                    listener.callBack(appCardData.id, CARD_CONTENT_REQUEST_SUCCESS);
                    
                    //更新缓存数据
                    appCardData.content = filterResult;
                    cacheMgr.setCache(appCardData.id, JSON.stringify(appCardData));
                    
                    //扫描返回的应用数组，获取应用的是否安装以及下载进度的信息，同时生成能够当前能够切换的20个应用
                    scanApps();
                    
                    //刷新视图
                    appCardDiv.refreshCardView();
                    
                    isAppCardRequested = true;
                    return;
                }
            }
            else { //其它错误统一处理
                console.log("error code is not 000000, error code is " + errorCode);
            }
        } 
        catch (e) {
            console.log("JSON parse exception");
        }
        
        //通知框架请求返回失败
        listener.callBack(appCardData.id, CARD_CONTENT_REQUEST_FAILE);
    }
    
    function handleRequestFail(){
        console.log("handleRequestFail()");
        //通知框架请求返回失败
        listener.callBack(appCardData.id, CARD_CONTENT_REQUEST_FAILE);
        
        isAppCardRequesting = false;
    }
    
    function startRequest(isNeedjudgeNet){
        console.log("startRequest(), isNeedjudgeNet is " + isNeedjudgeNet);
        if (isAppCardRequesting) {
            console.log("app card is requesting");
            return;
        }
        isAppCardRequesting = true;
        
        if (isNeedjudgeNet) {
            //获取网络状态
            var netWorkResultString = exec("SystemComponents", "isNetWorkAvailable", []);
            var netWorkResult = getResultMessage(netWorkResultString);
            console.log("netWorkResult is " + netWorkResult);
            //若网络未连接，则回调给框架请求失败
            if (netWorkResult == false) {
                //回调请求错误给框架
                listener.callBack(appCardData.id, CARD_CONTENT_REQUEST_FAILE);
                return;
            }
        }
        
        var requestOptions = {};
        if (baseInfo && baseInfo.businessRequestUrl) {
            var timeStr = new Date().valueOf();
            requestOptions.url = baseInfo.businessRequestUrl + "?c=1007&gz=0&v=1.0&t=" + timeStr;
        }
        else {
            listener.callBack(appCardData.id, CARD_CONTENT_REQUEST_FAILE);
            return;
        }
        
        requestOptions.onSuccess = handleRequestSuccess;
        requestOptions.onFail = handleRequestFail;
        
        //requestOptions.url = "http://192.168.77.214/Client_Test_developer/Default.aspx" + "?c=app&gz=0&v=1.0&t=2014";
        
        var base = organizeJsonBaseData(baseInfo);
        if (!base) {
            base = {};
        }
        var param = {
            "id": "app",
            "version": "" + appCardData.version,
            "extendParam": {}
        };
        var requestJson = {
            "cmd": "1007",
            "base": base,
            "param": param
        };
        console.log("requestJson is " + JSON.stringify(requestJson));
        
        requestOptions.data = JSON.stringify(requestJson);
        
        sendJsonAjaxRequest(requestOptions);
    }
    
    /**
     *创建单个推荐应用区域
     */
    function createRecommendAppDiv(recommendAppData, position){
        console.log("createRecommendAppDiv(), recommendAppData is " + JSON.stringify(recommendAppData));
        var downUrl = recommendAppData.downUrl;
        var imageUrl = recommendAppData.imgUrl;
		var bimgUrl = recommendAppData.bimgUrl;
		var count = recommendAppData.count;
        var name = recommendAppData.name;
        var packageName = recommendAppData.packageName;
        var isAutoOpen = (recommendAppData.autoOpen == 0 ? false : true);
        var entry = "card_app";
        var isInstalled = recommendAppData.isInstalled;
        var fileName = "";
        if (recommendAppData.fileName) {
            fileName = recommendAppData.fileName;
        }
        var size = "未知";
        if (recommendAppData.size) {
            size = recommendAppData.size;
        }
        console.log("fileName is " + fileName + ", size is " + size);
		
        
        // 外层的壳，屏蔽设置border对宽度计算的影响
		var recommendAppDivOut = document.createElement("div");
		recommendAppDivOut.className = "css_app_single_item_area_out";
		recommendAppDivOut.downUrl = downUrl;
		recommendAppDivOut.name = name;
		recommendAppDivOut.packageName = packageName;
		recommendAppDivOut.isAutoOpen = isAutoOpen;
		recommendAppDivOut.entry = entry;
		recommendAppDivOut.fileName = fileName;
		recommendAppDivOut.count = count;
		recommendAppDivOut.bimgUrl = bimgUrl;
		recommendAppDivOut.size = size;
		recommendAppDivOut.opCodeKey = opCodeKey.item + position; // position从1开始计数
		
		//整个卡片响应单击事件
		var move = false;
        recommendAppDivOut.addEventListener("touchstart", function(){
            move = false;
            shadowDiv.style.display = "block";
        }, false);
        recommendAppDivOut.addEventListener("touchmove", function(){
            if (moveOverCheck.check(event)) {
                move = true;
                shadowDiv.style.display = "none";
            }
        }, false);
        recommendAppDivOut.addEventListener("touchend", function(){
            if (!move) {
                if (clientVersion > VERSION_THRESHOLD) {
                    //调用下载能力接口进行处理
                    exec("DownloadComponents", "handleAppInstallAction", [recommendAppDivOut.downUrl, recommendAppDivOut.name, recommendAppDivOut.packageName, recommendAppDivOut.isAutoOpen, recommendAppDivOut.entry, recommendAppDivOut.fileName, recommendAppDivOut.size]);
                }
                else {
                    //调用浏览器打开downloadUrl
                    exec("UIComponents", 'openBrowser', ["", recommendAppDivOut.downUrl]);
                }
                //记录稽核日志及操作日志
                exec("WidgetMMPContainerComponents", "appenCardCpaLog", [cpaCodeKey]);
                exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.app, recommendAppDivOut.opCodeKey]);
            }
            shadowDiv.style.display = "none";
        }, false);
		
        // 单个推荐应用区域
		var recommendAppDiv = document.createElement("div");
		recommendAppDiv.className = "css_app_single_item_area";
		
		// 顶部
		var top = document.createElement("div");
		top.className = "css_app_single_item_area_top";
		
		// 应用介绍图
		var middle = document.createElement("img");
        middle.className = "css_app_single_item_area_middle";
		middle.src = bimgUrl;
		middle.onerror = function(){
			middle.src = res_path + "app_bimg_default.png";
            console.log("app big image load error!");
        };
        
        //应用小图标
		var smallIconOut = document.createElement("div");
        smallIconOut.className = "css_app_small_icon_out";
        var smallIcon = document.createElement("img");
        smallIcon.className = "css_app_small_icon";
		smallIcon.src = imageUrl;
        smallIcon.onerror = function(){
			smallIcon.style.display = "none";
            console.log("app image load error!");
        };
		smallIconOut.appendChild(smallIcon);
        top.appendChild(smallIconOut);
		
		var smallIconRightOut = document.createElement("div");
		smallIconRightOut.className = "css_app_small_icon_right_out";

		var appName = document.createElement("div");
		appName.className = "css_app_small_icon_right_out_top";
		appName.innerHTML = name;
		var appSizeAndDownloadTimes = document.createElement("div");
		appSizeAndDownloadTimes.className = "css_app_small_icon_right_out_bottom";
		var downloadTimes = getDownloadTimes(count);
		appSizeAndDownloadTimes.innerHTML = size + " " + downloadTimes;
		
		smallIconRightOut.appendChild(appName);
		smallIconRightOut.appendChild(appSizeAndDownloadTimes);
		top.appendChild(smallIconRightOut);
		
		var downloadBtnOutbox = document.createElement("div");
		downloadBtnOutbox.className = "download_btn_out_box";
		top.appendChild(downloadBtnOutbox);
		var downloadBtn = document.createElement("div");
		downloadBtn.className = "css_app_single_item_area_bottom_download_button";
		downloadBtn.innerHTML = "立即下载";        
		downloadBtnOutbox.appendChild(downloadBtn);
		recommendAppDiv.appendChild(top);
		recommendAppDiv.appendChild(middle);
        //添加遮盖div
        var shadowDiv = document.createElement("div");
        shadowDiv.className = "shadow_layer";
        recommendAppDiv.appendChild(shadowDiv);
        recommendAppDivOut.appendChild(recommendAppDiv);
        
        return recommendAppDivOut;
    }
    
    /**
     *创建底部的按钮区域
     */
    function createAppCardBottom(bottomButtonData){
        console.log("createAppCardBottom(), bottomButtonData is " + bottomButtonData);
        
        if (!bottomButtonData || bottomButtonData.length <= 0) {
            console.log("bottomButtonData is illegal!");
        }
        
        var appCardBottom = document.createElement("div");
		appCardBottom.className = "css_card_bottom_btn";
        appCardBottom.innerHTML = bottomButtonData[0].name;
        appCardBottom.detailUrl = bottomButtonData[0].url;
        
        //添加点击事件
        var cardBottomMove = false;
        var target = null;
        appCardBottom.addEventListener("touchstart", function(){
            event.stopPropagation();
            console.log("appCardBottom touchstart!");
            moveOverCheck.start(event);
            cardBottomMove = false;
            target = event.target;
            target.style.backgroundColor = "#efefef";
        }, false);
        appCardBottom.addEventListener("touchmove", function(){
            event.stopPropagation();
            console.log("appCardBottom touchmove!");
            if (moveOverCheck.check(event)) {
                cardBottomMove = true;
            }
            target.style.backgroundColor = "";
        }, false);
        appCardBottom.addEventListener("touchend", function(){
            event.stopPropagation();
            console.log("appCardBottom touchend!");
            if (false == cardBottomMove) {
                if (target) {
					target.style.backgroundColor = "";
                    if (target.detailUrl) {
                        exec("UIComponents", 'openBrowser', ["", target.detailUrl]);
                    }
                    
                    //记录稽核日志及操作日志
                    exec("WidgetMMPContainerComponents", "appenCardCpaLog", [cpaCodeKey]);
                    exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.app, opCodeKey.moreBtn]);
                }
            }
            target = null;
        }, false);
        
        appCardBottom.refreshBottomView = function(){
            var bottomButton = appCardData.content.bottomButton;
            if (!bottomButton || bottomButton.length <= 0) {
                return;
            }
            console.log("appCardBottom.refreshBottomView bottomButton is " + JSON.stringify(bottomButton));
            appCardBottom.innerHTML = bottomButton[0].name;
            appCardBottom.detailUrl = bottomButton[0].url;
        };
        
        return appCardBottom;
    }
    
    /**
     *创建推荐应用程序的滑动区域
     */
    function createRecommendAppsDiv(recomendAppsIndex){
        if (!recomendAppsIndex || recomendAppsIndex.length <= 0) {
            console.log("recomendAppsIndex is illegal!");
            return;
        }
        console.log("createRecommendAppsDiv(), recomendAppsIndex is " + JSON.stringify(recomendAppsIndex));
        
        var appSwipeFrame = document.createElement("div");
        appSwipeFrame.className = "css_app_swipe_frame";
        
        var appSwipeArea = document.createElement("div");
        appSwipeArea.className = "css_app_swipe_area";
        
        var spaceLeft = document.createElement("div");
        spaceLeft.className = "css_app_swipe_area_space";
        var spaceRight = document.createElement("div");
        spaceRight.className = "css_app_swipe_area_space";
        appSwipeArea.appendChild(spaceLeft);
        
        // 构建单个应用区域
        // 如果应用数量不够，修改SWIPE_SHOW_NUM的值
        if (recomendAppsIndex.length < SWIPE_SHOW_NUM) {
            SWIPE_SHOW_NUM = recomendAppsIndex.length;
        }
        for (var i = 0; i < SWIPE_SHOW_NUM; i++) {
            var recommendAppData = appCardData.content.apps[recomendAppsIndex[i]];
            var recommendAppDiv = createRecommendAppDiv(recommendAppData, i + 1);
            appSwipeArea.appendChild(recommendAppDiv);
			if (i < SWIPE_SHOW_NUM - 1){
				var marginRight = document.createElement("div");
				marginRight.className = "css_app_swipe_area_margin";
				appSwipeArea.appendChild(marginRight);
			}
        }
		SWIPE_SHOW_NUM = 4; // 还原
        appSwipeArea.appendChild(spaceRight);
        appSwipeFrame.appendChild(appSwipeArea);
        
        // 建立完之后，重新获取appSwipeArea等元素宽度，实现滑动框架的显示效果
        // 采用递归方式，控制递归次数
        var singleAppWidth = 0; // 每个应用区域所占宽度
        var spaceWidth = 0; // 左右两边边距的宽度
        var marginRightWidth = 0;
        var recursionTimes = 0; // 递归执行次数
        function setSwipeAreaWidth(){
            var nodesLength = appSwipeArea.childNodes.length;
            console.log("setSwipeAreaWidth appSwipeArea nodesLength: " + nodesLength);
            
            if (nodesLength >= 4) { // 至少包含一个应用，右边margin和两边的边距
                singleAppWidth = appSwipeArea.childNodes[1].clientWidth;
                spaceWidth = appSwipeArea.childNodes[0].clientWidth;
				marginRightWidth = appSwipeArea.childNodes[2].clientWidth;
                recursionTimes++;
                
                if (recursionTimes >= 100 || (singleAppWidth != 0 && spaceWidth != 0 && marginRightWidth != 0)) {
                    if (singleAppWidth != 0 && spaceWidth != 0 && marginRightWidth != 0) {
                        var totalWidth = 0.5 * (nodesLength - 1) * singleAppWidth + 0.5 * (nodesLength - 3) * marginRightWidth + 2 * spaceWidth + 'px';
                        console.log("setSwipeAreaWidth totalWidth: " + totalWidth);
                        // 重新对宽度赋值
                        appSwipeArea.style.width = totalWidth;
                    }
                    console.log("setSwipeAreaWidth singleAppWidth: " + singleAppWidth + ", obtained");
                    console.log("setSwipeAreaWidth spaceWidth: " + spaceWidth + ", obtained");
					console.log("setSwipeAreaWidth marginRightWidth: " + marginRightWidth + ", obtained");
                    console.log("setSwipeAreaWidth recursionTimes: " + recursionTimes + ", last");
                    return;
                }
            }
            
            setTimeout(function(){
                // 递归执行
                setSwipeAreaWidth();
            }, 15);
        }
        
        setSwipeAreaWidth(); // 函数内部执行了setTimeout
        
        // 记录滑动日志
        // 只有发生有效滑动即页面切换时才会记录，不区分滑动方向
        appSwipeArea.appendSwipeLog = function(){
            exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.app, "swipeAppPage"]);
        }
        
		
        /**
         * 对外提供的刷新界面的接口
         */
        appSwipeFrame.refreshRecommendAppsView = function(refreshAppsDataIndex){
            console.log("recommendAppsDiv.refreshRecommendAppsView()");
			appSwipeArea.innerHTML = "";

            var spaceLeft2 = document.createElement("div");
            spaceLeft2.className = "css_app_swipe_area_space";
            var spaceRight2 = document.createElement("div");
            spaceRight2.className = "css_app_swipe_area_space";
            appSwipeArea.appendChild(spaceLeft2);
            
            // 构建单个应用区域
            // 如果应用数量不够，修改SWIPE_SHOW_NUM的值
            if (refreshAppsDataIndex.length < SWIPE_SHOW_NUM) {
                SWIPE_SHOW_NUM = refreshAppsDataIndex.length;
            }
            for (var i = 0; i < SWIPE_SHOW_NUM; i++) {
                var appData = appCardData.content.apps[refreshAppsDataIndex[i]];
                var appDiv = createRecommendAppDiv(appData, i + 1);
                appSwipeArea.appendChild(appDiv);
                if (i < SWIPE_SHOW_NUM - 1) {
                    var marginRight2 = document.createElement("div");
                    marginRight2.className = "css_app_swipe_area_margin";
                    appSwipeArea.appendChild(marginRight2);
                }
            }
			SWIPE_SHOW_NUM = 4; // 还原
            appSwipeArea.appendChild(spaceRight2);
			setSwipeAreaWidth();
        };
		
        // 构造对象传入应用推荐滑动框架
        var obj = {
            "swipe_frame": appSwipeFrame,
            "swipe_area": appSwipeArea,
        };
        var BS = new appSwipe(obj);
        
        return appSwipeFrame;
    }
    
    /**
     *创建天气卡片的title区域
     */
    function createAppCardTitle(appData){
        console.log("createAppCardTitle, appData is " + JSON.stringify(appData));
        
        var appCardTitle = document.createElement("div");
        appCardTitle.className = "css_card_title_bar";
        
        //左边的标题区域
        var leftTitle = document.createElement("div");
        leftTitle.className = "css_card_title_text";
        leftTitle.innerHTML = appCardData.name;
        
        appCardTitle.appendChild(leftTitle);
        return appCardTitle;
    }
    
    function createAppCardView(){
        console.log("createAppCardView()");
        
        var appData = null;
        if (appCardData) {
            appData = appCardData.content;
        }
        console.log("appData is " + JSON.stringify(appData));
        
        var appArea = document.createElement("div");
        appArea.className = "css_card";
        
        // 创建应用卡片的title区域
        var appCardTitle = createCardMainTitle(appCardData.id, appCardData.name, appCardData.caseList[0], titleResponseListener, appCardData.caseRedDot);
        if (appCardTitle) {
            appArea.appendChild(appCardTitle);
        }
        
        //创建应用卡片的推荐应用滑动区域
        var appCardRecommendApps = createRecommendAppsDiv(currentShowAppsIndex);
        appArea.appendChild(appCardRecommendApps);
        
        //创建底部换一换和更多应用区域
        var appCardBottom = createAppCardBottom(appData.bottomButton);
        appArea.appendChild(appCardBottom);
        
        //创建底部的阴影区
        var appCardBottomShadow = document.createElement("div");
        appCardBottomShadow.className = "css_card_bottom_shadow";
        appArea.appendChild(appCardBottomShadow);
        
        appCardDiv.refreshCardView = function(){
            console.log("appCardDiv.refreshCardView");
            //刷新推荐应用的视图
            appCardRecommendApps.refreshRecommendAppsView(currentShowAppsIndex);
            //刷新应用底部的视图
            appCardBottom.refreshBottomView();
        };
        
        //保存变量
        appCard = appArea;
        
        if (appCardDiv) {
            appCardDiv.appendChild(appArea);
            appCardDiv.showContentPage = showContentPage;
        }
        
        //设置已经创建的标识位为true
        isAppCardCreate = true;
    }
    
    function onResourceLoadFinish(){
        console.log("onResourceLoadFinish(), appCardData is " + JSON.stringify(appCardData));
        
        //获取基本信息以及客户端的版本号
        baseInfo = getBseInfo();
        if (baseInfo && baseInfo.base) {
            var clientVersionString = String(baseInfo.base.clientver);
            if (clientVersionString) {
                var clientVersionArray = clientVersionString.split(".");
                if (clientVersionArray && clientVersionArray.length >= 3) {
                    clientVersion = parseInt(clientVersionArray[2]);
                    console.log("clientVersion is " + clientVersion);
                }
            }
        }
        
        
        //扫描应用数组，获取应用的是否安装以及下载进度的信息，同时生成能够当前能够切换的20个应用
        scanApps();
        
        //创建应用卡片视图
        createAppCardView();
        
        //注册下载监听器
        //exec("DownloadComponents", "registerDownloadListener",[1]);
        
        //请求应用数据
        if (1 == requestCondition) {
            startRequest(false);
        }
        else {
            if (2 == requestCondition) {
                startRequest(true);
            }
            else {
                //没有网络，不需要请求，但是需要回调
                listener.callBack(appCardData.id, CARD_CONTENT_REQUEST_FAILE);
            }
        }
    }
    
    function getInstalledAndDownloadInfo(appsIndex){
        console.log("getInstalledAndDownloadInfo(), appsIndex is " + appsIndex);
        var packageNames = new Array();
        var installedAppsIndex = new Array();
        var uninstalledAppsIndex = new Array();
        for (var i = 0; i < appsIndex.length; i++) {
            var appData = appCardData.content.apps[appsIndex[i]];
            packageNames.push(appData.packageName);
        }
        
        //调用接口获取应用是否安装信息
        var appInstalledResult = getResultMessage(exec("AppComponents", "isAppInstalled", [packageNames]));
        console.log("appInstalledResult is " + JSON.stringify(appInstalledResult));
        for (var i = 0; i < packageNames.length; i++) {
            var index = appsIndex[i];
            if (true == appInstalledResult[packageNames[i]]) {//安装
                installedAppsIndex.push(index);
                appCardData.content.apps[index].isInstalled = true;
            }
            else { //未安装
                uninstalledAppsIndex.push(index);
                appCardData.content.apps[index].isInstalled = false;
            }
        }
        
        //获取未安装应用程序的下载进度信息
        //getAppsDownloadInfo(uninstalledAppsIndex);
    }
    
    /**
     * 扫描应用数组，获取应用的是否安装以及下载进度的信息，同时生成能够当前能够切换的20个应用
     * @returns {Array}
     */
    function scanApps(){
        //初始化操作,清空数组
        installedAppIndex.splice(0, installedAppIndex.length);
        uninstalledAppIndex.splice(0, uninstalledAppIndex.length);
        
        //遍历apps数组获取所有应用的包名
        allPackageNames = getAllAppPackageNames();
        console.log("allPackageNames is " + allPackageNames);
        
        if (clientVersion > VERSION_THRESHOLD) {
            //调用接口检查应用程序是否已安装
            var appInstalledResult = getResultMessage(exec("AppComponents", "isAppInstalled", [allPackageNames]));
            console.log("isAppInstalled result is " + JSON.stringify(appInstalledResult));
            for (var index = 0; index < allPackageNames.length; index++) {
                if (true == appInstalledResult[allPackageNames[index]]) {
                    installedAppIndex.push(index);
                    appCardData.content.apps[index].isInstalled = true;
                }
                else {
                    uninstalledAppIndex.push(index);
                    appCardData.content.apps[index].isInstalled = false;
                }
            }
            console.log("installedAppIndex is " + installedAppIndex);
            console.log("uninstalledAppIndex is " + uninstalledAppIndex);
            
            //根据应用程序是否已安装的情况，去获取未安装应用程序的下载进度
            //getAppsDownloadInfo(uninstalledAppIndex);
            
            //获取20个用来切换展示的应用
            canSwitchAppsIndex = getSwitchAppsIndex();
        }
        else {
            //旧版本，用来切换的20个app就是所有app中的前20个
            canSwitchAppsIndex = getSwitchAppsIndex();
        }
		
		currentShowAppsIndex = canSwitchAppsIndex.slice(0, SWIPE_SHOW_NUM);
        console.log("canSwitchAppsIndex is " + canSwitchAppsIndex + ", currentShowAppsIndex is " + currentShowAppsIndex);
    }
    
    /**
     * 获取用于切换展示的20个应用
     * @returns {Array}
     */
    function getSwitchAppsIndex(){
        var switchApps = [];
		
        var totalNum = allPackageNames.length; // 应用总数目，肯定不小于4
        console.log("appCard totalNum (allPackageNames.length) is " + totalNum);        
        if (totalNum > MAX_PAGE_NUM * PAGE_SHOW_NUM) { // 设置最大长度限制
            totalNum = MAX_PAGE_NUM * PAGE_SHOW_NUM;
            console.log("appCard totalNum (max) is " + totalNum);
        }
        if (clientVersion > VERSION_THRESHOLD) {
            //将未安装的应用添加到可切换的应用数组中
            for (var i = 0; i < uninstalledAppIndex.length; i++) {
                if (switchApps.length >= totalNum) {
                    break;
                }
                switchApps.push(uninstalledAppIndex[i]);
            }
			
            // 若可供切换的应用数目不足SWIPE_SHOW_NUM = 4个，则用已安装的应用进行补充
            var remainNum = SWIPE_SHOW_NUM - switchApps.length;
            if (installedAppIndex.length > 0 && remainNum > 0) {
                for (var i = 0; i < remainNum; i++) {
                    switchApps.push(installedAppIndex[i]);
                }
            }
        }
        else { //1909及之前的版本直接获取前totalNum个应用作为切换展示的应用
            for (var i = 0; i < totalNum; i++) {
                switchApps.push(i);
            }
        }
        
        return switchApps;
    }
    
    /**
     * 获取所有推荐应用的包名
     * @returns {Array}
     */
    function getAllAppPackageNames(){
        var packageNames = [];
        var apps = appCardData.content.apps;
        for (var i = 0; i < apps.length; i++) {
            packageNames.push(apps[i].packageName);
        }
        return packageNames;
    }
    
    /**
     * 获取未安装的应用
     */
    function getUninstallApps(uninstalledAppsIndex){
        var uninstallPackage = [];
        if (allPackageNames && allPackageNames.length > 0) {
            for (var i = 0; i < uninstalledAppsIndex.length; i++) {
                uninstallPackage.push(allPackageNames[uninstalledAppsIndex[i]]);
            }
        }
        return uninstallPackage;
    }
	
    /**
     * 产生随机数
     */
    function getRandomNum(min, max){
        var range = max - min;
        var rand = Math.random(); // 0~1之间的随机数
        return (min + Math.round(rand * range - 0.1)); // 四舍五入
    }
	
	/**
     * 下载次数
     */
    function getDownloadTimes(numStr){
        console.log("getDownloadTimes, numStr is " + numStr);
        var downlodaTimesText = "";
        try {
            var numInt = parseInt(numStr);
            
            if (isNaN(numInt)) {
                return downlodaTimesText;
            }
            
            if (numInt < 10000) {
                downlodaTimesText += numInt;
            }
            else {
                // 下载次数大于等于1万时，以万做单位，保留小数点后一位
                var numIntWan = (numInt / 10000).toFixed(1);
                downlodaTimesText += String(numIntWan) + "万";
                
            }
        } 
        catch (e) {
            console.log("get downlodaTimesText, exception is " + e);
        }
        return downlodaTimesText + "次下载";
    }
	
	
    function createAppCard(initNetState){
        var appCardId = appCardData.id;
        
        //读取缓存内容
        var cacheAppString = cacheMgr.getCache(appCardId);
        console.log("cacheAppString is " + cacheAppString);
        if (!cacheAppString) { //缓存为空，则使用默认数据
            cacheMgr.setCache(appCardId, JSON.stringify(appCardData));
        }
        else {
            if (typeof cacheAppString == "string") {
                try {
                    var appCacheCardData = JSON.parse(cacheAppString);
                    //检查缓存的有效性，即比较版本号
                    //若版本号相同的话，则缓存有效，直接使用缓存
                    //若版本号不相同的话，则缓存无效，这时需取缓存的“show”字段
                    if (appCacheCardData.version == appCardData.version) {
                        appCardData.show = appCacheCardData.show;
                        appCardData.caseRedDot = appCacheCardData.caseRedDot;
                        if (appCacheCardData.content) {
                            appCardData.content = appCacheCardData.content;
                        }
                    }
                    else {
                        appCardData.show = appCacheCardData.show; //保留卡片是否需要显示
                        cacheMgr.setCache(appCardId, JSON.stringify(appCardData));
                    }
                } 
                catch (e) {
                    cacheMgr.setCache(appCardId, JSON.stringify(appCardData));
                    console.log("JSON parse exception");
                }
            }
        }
        
        //若卡片可以显示，则加载资源并创建视图
        console.log("appCardData is " + JSON.stringify(appCardData));
        if (appCardData && appCardData.show) {
            if (initNetState) {
                requestCondition = 1;
            }
            else {
                requestCondition = 0;
            }
            //加载css资源文件
            xcssParser.parseStyleFile("./app/business.x.css", onResourceLoadFinish);
        }
        else {
            //卡片不需要显示时，也需要回调给框架请求成功
            listener.callBack(appCardData.id, CARD_CONTENT_NO_REQUEST);
            requestCondition = 2;
        }
    }
    
    function createAppCardFrame(cardDefaultData, initNetState){
        console.log("createAppCardFrame(), cardDefalutData is " + JSON.stringify(cardDefaultData));
        appCardData = cardDefaultData;
        
        appCardDiv = document.createElement("div");
        appCardDiv.id = appCardData.id;
        
        createAppCard(initNetState);
        
        appCardDiv.updateData = function(){
            console.log("appCardDiv.updateData(), appCardData.show is " + appCardData.show);
            if (appCardData.show) {
                startRequest(false);
            }
            else {
                if (!isAppCardRequesting) {
                    listener.callBack(appCardData.id, CARD_CONTENT_NO_REQUEST);
                }
            }
        };
        
        //更新卡片是否显示的状态
        appCardDiv.updateState = function(state){
            console.log("appCardDiv.updateCardState, state is " + state);
            if (true == state || "true" == state) { //显示卡片
                if (isAppCardCreate) { //应用卡片视图已创建
                    console.log("isAppCardRequested is " + isAppCardRequested);
                    appCardDiv.style.display = "block";
                    if (!isAppCardRequested) {
                        //未成功请求过，不仅需要显示视图，而且还需进行请求
                        if (!isAppCardRequesting) {
                            startRequest();
                        }
                    }
                }
                else { //应用卡片视图未创建
                    xcssParser.parseStyleFile("./app/business.x.css", onResourceLoadFinish);
                }
            }
            else 
                if (false == state || "false" == state) { //不显示卡片
                    appCardDiv.style.display = "none";
                }
            
            //更新缓存
            appCardData.show = state;
            cacheMgr.setCache(appCardData.id, JSON.stringify(appCardData));
        };
        
        appCardDiv.date_type = "card";
        appCardDiv.initShowState = appCardData.show;
        
        return appCardDiv;
    }
    
    function operateCallBack(callBackId){
        switch (callBackId) {
            case 0://取消
                if (zdOffDom) {
                    zdOffDom.style.display = "none";
                }
                appCard.style.display = "block";
                break;
            case 1://置顶
                if (zdOffDom) {
                    zdOffDom.style.display = "none";
                }
                appCard.style.display = "block";
                stickTopCardByDom(appCardDiv);
                break;
            case 2://关闭
                //显示关闭确认弹框
                //showCloseConfirmDialogById(appCardData.id);
                break;
                
            default:
                break;
        }
    }
    function speechCaseCallBack(){
        if (zdOffDom) {
            zdOffDom.style.display = "none";
        }
        if (speechCaseDom) {
            speechCaseDom.style.display = "none";
        }
        appCard.style.display = "block";
    }
    function titleResponseListener(responseId){
        switch (responseId) {
            case 0://点击说法示例区域
                if (appCardData && appCardData.caseRedDot) {
                    appCardData.caseRedDot = false;
                    cacheMgr.setCache(appCardData.id, JSON.stringify(appCardData));
                }
                if (zdOffDom) {
                    zdOffDom.style.display = "none";
                }
                
                appCard.style.display = "none";
                if (speechCaseDom) {
                    speechCaseDom.style.display = "block";
                }
                else {
                    speechCaseDom = createSpeechCaseView(appCardData.id, appCardData.name, appCardData.caseList);
                    if (speechCaseDom) {
                        speechCaseDom.setCancelListener(speechCaseCallBack);
                        appCardDiv.appendChild(speechCaseDom);
                    }
                }
                break;
            case 1://点击置顶箭头按钮
                if (speechCaseDom) {
                    speechCaseDom.style.display = "none";
                }
                appCard.style.display = "none";
                
                if (zdOffDom) {
                    zdOffDom.style.display = "block";
                }
                else {
                    zdOffDom = createOperationView(appCardData.id, appCardData.name, appCardData.id);
                    if (zdOffDom) {
                        zdOffDom.setOpListener(operateCallBack);
                        appCardDiv.appendChild(zdOffDom);
                    }
                }
                break;
            case 2://刷新，暂时没有实现
                break;
            default:
                break;
        }
    }
    function showContentPage(){
        if (zdOffDom) {
            zdOffDom.style.display = "none";
        }
        if (speechCaseDom) {
            speechCaseDom.style.display = "none";
        }
        appCard.style.display = "block";
    }
    
    return createAppCardFrame;
});
