define(function() {
	var res_path = "./weather/res/";
	var weather_refresh_img = res_path + "ic_mainpage_refresh.png";
	var weather_default_img = res_path + "weather_default.png";
	var weather_schedule_guide_clear = res_path + "ic_mainpage_operations_close.png";
	
	var weatherCardData = null; //天气卡片数据
	var weatherCardDiv = null;
	var isWeatherCardCreate = false; //标识天气卡片视图是否已经创建
	var isWeatherCardRequested = false; //标识天气卡片是否请求过
	var isWeatherCardRequesting = false; //标识天气卡片是否正在请求
	var contentRotateId = null; //内容区域动画id
	var titleRotateId = null; //标题区域动画id
	var isHasCacheData = false; //标识是否具有缓存数据
	var opCodeKey = "";//记录操作日志的key
	
    var requestCondition = 0;//0:无网络，不需要请求;1:不判断网络，直接请求;2：判断网络，再请求 
    var requestTrigger = 0;//0:卡片本身触发 ;1:框架触发

    var zdOffDom = null;
    var speechCaseDom = null;
    var weatherCard = null;
	
	//各种状态
	var loadFailState = {
	    LOCATING:"locating",
	    LOADING:"loading",
	    LOAD_FAIL:"loadFail",
	    NOT_SUPPORT:"notSupport",
	    NONE_LOCATION:"noneLocation",
	    DISAPPEAR:"disappear"
	};
	
	//加载错误提示语
	var loadFailTips = {
	    LOCATING:"数据加载中，请稍候...",
	    LOAD_FAIL:"加载失败，点击重试",
	    NOT_SUPPORT:"暂不支持该地区的天气查询",
	    NONE_LOCATION:"定位失败，点击重试"
	};
	
	function checkWeatherResult(weatherResult) {
		console.log("checkWeatherResult, weatherResult is " + JSON.stringify(weatherResult));
		if (!weatherResult) {
			return false;
		}
		if (!weatherResult.date || !weatherResult.city || !weatherResult.condition 
		    || !weatherResult.low || !weatherResult.high || !weatherResult.bottomButton
            || !weatherResult.bottomButton.url || !weatherResult.bottomButton.name) {
		    return false;
		}
		if (!weatherResult.condition.description || !weatherResult.condition.image) {
			return false;
		}
		return true;
	}
	
	function filterRequestResult(requestResult) {
	    if (!requestResult) {
	        return null;
	    }
	    var filterResult = {};
	    if (requestResult.content && requestResult.content.weathers) {
	        if (requestResult.content.weathers.length > 0) {
	            var weather = requestResult.content.weathers[0];
	            if (weather) {
	                if (weather.city) {
	                    filterResult.city = weather.city;
	                }
	                if (weather.last_update && weather.last_update.datetime) {
	                    filterResult.date = weather.last_update.datetime.date + " " + weather.last_update.datetime.time;
	                }
	                if (weather.forecasts && weather.forecasts.length > 0) {
	                    var forecast = weather.forecasts[0];
	                    if (forecast) {
	                        if (forecast.low) {
	                            filterResult.low = forecast.low;
	                        }
	                        if (forecast.high) {
	                            filterResult.high = forecast.high;
	                        }
	                        if (forecast.pm25) {
	                            filterResult.pm25 = {};
	                            if (forecast.pm25.num) {
	                                filterResult.pm25.num = forecast.pm25.num;
	                            }
	                            if (forecast.pm25.desc) {
	                                filterResult.pm25.desc = forecast.pm25.desc;
	                            }
	                        }
	                        if (forecast.condition && forecast.condition.description && forecast.condition.image_url) {
	                            filterResult.condition = {};
	                            filterResult.condition.description = forecast.condition.description;
	                            filterResult.condition.image = forecast.condition.image_url;
	                        }
	                    }
	                }
	            }
	        }
	    }
	    if (requestResult.content && requestResult.content.bottomButton && requestResult.content.bottomButton.length > 0) {
	        var bottomButton = requestResult.content.bottomButton[0];
	        if (bottomButton && bottomButton.name && bottomButton.url) {
	            filterResult.bottomButton = {};
	            filterResult.bottomButton.name = bottomButton.name;
	            filterResult.bottomButton.url = bottomButton.url;
	        }
	    }
	    
	    return filterResult;
	}

    /**
     * 判断数据是否有改变
     */
    function isWeatherDataChanged(oldData, newData) {
        if (!oldData) {
            return true;
        }
        console.log("isWeatherDataChanged(), oldData is " + JSON.stringify(oldData) + ", newData is " + JSON.stringify(newData));

        if (oldData.city != newData.city || oldData.low != newData.low || oldData.high != newData.high) {
            return true;
        }

        var oldPM25 = oldData.pm25;
        var newPM25 = newData.pm25;
        if (oldPM25 && newPM25) {
            if (oldPM25.num != newPM25.num || oldPM25.desc != newPM25.desc) {
                return true;
            }
        } else {
            return true;
        }

        var oldCondition = oldData.condition;
        var newCondition = newData.condition;
        if (oldCondition.description != newCondition.description || oldCondition.image != newCondition.image) {
            return true;
        }

        var oldButtomData = oldData.bottomButton;
        var newButtomData = newData.bottomButton;
        if (oldButtomData.name != newButtomData.name || oldButtomData.url != newButtomData.url) {
             return true;
        }

        return false;
    }

	function handleWeatherRequestData(requestResult) {
		console.log("handleWeatherRequestData, requestResult is " + requestResult);
        isWeatherCardRequesting = false;
        
		/**
		 * 根据返回不同的状态进行不同处理
		 */
		try {
		    var requestResultJson = JSON.parse(requestResult);
		    var status = requestResultJson.status;
            var errorCode = requestResultJson.errorCode;
            console.log("status is " + status + ", errorCode is " + errorCode);
            if ("000000" == errorCode) { //数据成功返回
                //解析返回结果
                var filterResult = filterRequestResult(requestResultJson);
                console.log("filterResult is " + JSON.stringify(filterResult));
                if (checkWeatherResult(filterResult)) {
                    //回调给框架请求成功返回
                    listener.callBack(weatherCardData.id, CARD_CONTENT_REQUEST_SUCCESS);

                    if (isWeatherDataChanged(weatherCardData.content, filterResult)) {
                        //刷新天气视图
                        weatherCardDiv.refreshWeatherView(filterResult, true);
                    } else {
                        //刷新天气视图
                        weatherCardDiv.refreshWeatherView(filterResult, false);
                    }

                    //更新缓存数据
                    weatherCardData.content = filterResult;
                    cacheMgr.setCache(weatherCardData.id, JSON.stringify(weatherCardData));

                    isHasCacheData = true;
                    isWeatherCardRequested = true;
                    
                    //保存天气详情页面的地址
                    exec("SystemComponents", "setSetting", ["010001", weatherCardData.content.bottomButton.url, 5]);
                    return;
                } else {
                    weatherCardDiv.show(loadFailState.LOAD_FAIL, loadFailTips.LOAD_FAIL);
                }
            } else if ("550002" == errorCode) {//返回暂不支持该地区天气查询的错误
                weatherCardDiv.show(loadFailState.NOT_SUPPORT, requestResultJson.desc);
            } else { //其它错误统一处理
                weatherCardDiv.show(loadFailState.LOAD_FAIL, loadFailTips.LOAD_FAIL);
            }  
		} catch(e) {
            weatherCardDiv.show(loadFailState.LOAD_FAIL, loadFailTips.LOAD_FAIL);
		    console.log("JSON parse exception");
		}

        //回调给框架请求失败返回
        listener.callBack(weatherCardData.id, CARD_CONTENT_REQUEST_FAILE);
	}
	
	function handleWeatherRequestFail() {
	    console.log("handleWeatherRequestFail()");
        isWeatherCardRequesting = false;
	    //通知框架请求返回失败
        listener.callBack(weatherCardData.id, CARD_CONTENT_REQUEST_FAILE);

	    //显示出错界面
	    weatherCardDiv.show(loadFailState.LOAD_FAIL, loadFailTips.LOAD_FAIL);
	}

	function startRequestWeather(isNeedjudgeNet) {
		if(isWeatherCardRequesting){
			console.log("weather card is requesting");
			return;
		}
	    console.log("startRequestWeather()");
		var requestOptions = {};

		var baseInfo = getBseInfo();
		
		var netWorkResult = true;
		if(!(false == isNeedjudgeNet)) {
			 //获取网络状态
	        var netWorkResultString = exec("SystemComponents", "isNetWorkAvailable", []);
	        netWorkResult = getResultMessage(netWorkResultString);
		}
		
        console.log("netWorkResult is " + netWorkResult);
        
		if(!netWorkResult){
			if(0 == requestTrigger) {
		    	exec("UIComponents","showToast",["网络无连接，请检测网络设置",1]);
			}
		}
        //在有网的环境下，判断位置信息是否为空，为空则展示定位失败的出错界面
        if (netWorkResult == true && !baseInfo.address) {
            //回调请求错误给框架
            listener.callBack(weatherCardData.id, CARD_CONTENT_REQUEST_FAILE);
            weatherCardDiv.show(loadFailState.NONE_LOCATION, loadFailTips.NONE_LOCATION);
            isWeatherCardRequesting = false;
            return;
        }
        if (baseInfo && baseInfo.businessRequestUrl) {
            var timeStr = new Date().valueOf();
            requestOptions.url = baseInfo.businessRequestUrl + "?c=1007&gz=0&v=1.0&t=" + timeStr;
        } else {
            listener.callBack(weatherCardData.id, CARD_CONTENT_REQUEST_FAILE);
            weatherCardDiv.show(loadFailState.LOAD_FAIL, loadFailTips.LOAD_FAIL);
            isWeatherCardRequesting = false;
            return;
        }

        requestOptions.onSuccess = handleWeatherRequestData;
        requestOptions.onFail = handleWeatherRequestFail;

		//requestOptions.url = "http://192.168.77.214/Client_Test_developer/Default.aspx" + "?c=m1002&gz=0&v=1.0&t=2014";

		var base = organizeJsonBaseData(baseInfo);
        if(!base){
            base = {};
        }
		var param = {
			"id" : "weather",
			"version" : "" + weatherCardData.version,
            "extendParam":{}
		};
		var requestJson = {
			"cmd" : "1007",
			"base" : base,
			"param" : param
		};
		console.log("requestJson is " + JSON.stringify(requestJson));

		requestOptions.data = JSON.stringify(requestJson);

		sendJsonAjaxRequest(requestOptions);

        isWeatherCardRequesting = true;
	}

	/**
	 *创建天气卡片的title区域
	 */
	function createWeatherCardTitle(weatherData) {
		console.log("createhWeatherCardTitle, weatherData is " + JSON.stringify(weatherData));

		var weatherCardTitle = document.createElement("div");
		weatherCardTitle.className = "css_card_title_bar";

		//左边的标题区域
		var leftTitle = document.createElement("div");
		leftTitle.className = "css_card_title_text";
        leftTitle.innerText = weatherCardData.name;

		//更新时间区域
		var updateTime = document.createElement("div");
		updateTime.className = "css_weather_update_time_div";
		updateTime.addEventListener("click", function(){}, false);

		//右边的刷新按钮区域
		var refreshButton = document.createElement("div");
		refreshButton.className = "css_weather_refresh_button";

		var loadingImg = document.createElement("img");
        loadingImg.className = "css_weather_title_loading_img";
        loadingImg.src = frame_loading_img;
        loadingImg.style.display = "none";

		var refreshImg = document.createElement("img");
		refreshImg.className = "css_weather_refresh_img";
		refreshImg.src = weather_refresh_img;

		refreshButton.appendChild(loadingImg);
		refreshButton.appendChild(refreshImg);

		if (weatherData && weatherData.date) {
			updateTime.innerText = "更新于:" + weatherData.date;
			refreshButton.style.display = "block";
		} else {
			updateTime.innerText = "";
			refreshButton.style.display = "none";
		}

		function startLoading() {
            loadingImg.style.display = "block";
			refreshImg.style.display = "none";
			refreshButton.style.display = "block";
			updateTime.innerText = "正在更新...";

			//停止加载动画
			if (titleRotateId) {
				stopRotateAnimation(titleRotateId);
			}

			//开始动画
			titleRotateId = startRotateAnimation(loadingImg);
		}

		function stopLoading() {
            loadingImg.style.display = "none";
			refreshImg.style.display = "block";
			refreshButton.style.display = "block";

			//停止加载动画
			if (titleRotateId) {
				stopRotateAnimation(titleRotateId);
				titleRotateId = null;
			}
		}

		var refreshButtonMove = false;
		refreshButton.addEventListener("touchstart", function() {
			event.stopPropagation();
			console.log("refreshButtonMove touchstart!");
			moveOverCheck.start(event);
			refreshButtonMove = false;
		}, false);
		refreshButton.addEventListener("touchmove", function() {
			event.stopPropagation();
			console.log("refreshButtonMove touchmove!");
			if (moveOverCheck.check(event)) {
				refreshButtonMove = true;
			}
		}, false);
		refreshButton.addEventListener("touchend", function() {
			event.stopPropagation();
			console.log("refreshButtonMove touchend!");
			if (false == refreshButtonMove) {
                if (event.target == refreshImg) {
                    if (!isWeatherCardRequesting) {
                    	startLoading();
                        //开始请求天气数据
                        requestTrigger = 0;
                        startRequestWeather();
                    }
                    
                    opCodeKey = "weatherTitleRefreshButton";
                    exec("WidgetMMPContainerComponents", "appenCardCpaLog", ["weather"]);
                    exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.weather, opCodeKey]);
                }
			}
		}, false);

		//刷新title
		weatherCardTitle.refreshTitle = function(updateTimeData) {
		    console.log("weatherCardTitle.refreshTitle(), updateTime is " + updateTimeData);
			updateTime.innerText = "更新于:" + updateTimeData;
		};

		//开始动画
		weatherCardTitle.startLoading = function() {
			startLoading();
		};

		//停止动画
		weatherCardTitle.stopLoading = function() {
			stopLoading();
		};

		weatherCardTitle.appendChild(leftTitle);
		weatherCardTitle.appendChild(updateTime);
		weatherCardTitle.appendChild(refreshButton);
		return weatherCardTitle;
	}

	/**
	 * 创建天气卡片的content区域
	 */
	function createWeatherCardContent(weatherData) {
		console.log("createWeatherCardContent, weatherData is " + JSON.stringify(weatherData));
		var weatherCardContent = document.createElement("div");
		weatherCardContent.className = "css_weather_card_content_div";

		//左边的天气显示区域
		var leftContentDiv = document.createElement("div");
		leftContentDiv.className = "css_weather_left_content_div";

		//温度显示区域
		var weatherTemp = document.createElement("div");
		weatherTemp.className = "css_weather_temp";

		//天气情况描述区域
		var weatherDescription = document.createElement("div");
		weatherDescription.className = "css_weather_description";

		//PM 2.5描述区域
		var weatherPm25 = document.createElement("div");
		weatherPm25.className = "css_weather_pm25";

		leftContentDiv.appendChild(weatherTemp);
		leftContentDiv.appendChild(weatherDescription);
		leftContentDiv.appendChild(weatherPm25);

		//右边的天气图片显示区域
		var rightImgDiv = document.createElement("div");
		rightImgDiv.className = "css_weather_right_img_div";
		
		var weatherDefaultImg = document.createElement("div");
		weatherDefaultImg.className = "css_weather_default_img";

		var weatherImg = document.createElement("img");
		weatherImg.className = "css_weather_right_img";
        weatherImg.onload = function() {
            console.log("weatherImg onload sucess!");
            weatherDefaultImg.style.display = "none";
            weatherImg.style.display = "block";
        };
        rightImgDiv.appendChild(weatherDefaultImg);
		rightImgDiv.appendChild(weatherImg);

		//有缓存数据时，显示缓存数据；无缓存数据时，显示空白
		if (checkWeatherResult(weatherData)) {
			weatherCardContent.style.visibility = "visible";
			weatherTemp.innerText = weatherData.low + "°/" + weatherData.high + "°";
			var cityInfo = weatherData.city;
			if (cityInfo != null && cityInfo != undefined && cityInfo != "") {
                if (cityInfo.length > 9) {
                    cityInfo = cityInfo.substring(0, 7) + "...";
                }
            }
			weatherDescription.innerText = cityInfo + "  " + weatherData.condition.description;
            if (weatherData.pm25 && weatherData.pm25.num && weatherData.pm25.desc) {
                weatherPm25.innerText = "空气质量: " + weatherData.pm25.desc;
            } else {
                weatherPm25.innerText = "空气质量: 无";
            }
			rightImgDiv.style.display = "block";
			weatherDefaultImg.style.display = "block";
            weatherImg.style.display = "none";
			weatherImg.src = weatherData.condition.image;
		} else {
			console.log("weatherData is null");
			weatherCardContent.style.visibility = "hidden";
		}

        var weatherCardContentMove = false;
        weatherCardContent.addEventListener("touchstart", function() {
            event.stopPropagation();
            console.log("weatherCardContent touchstart!");
            moveOverCheck.start(event);
            weatherCardContent.style.backgroundColor = "#efefef";
            weatherCardContentMove = false;
        }, false);
        weatherCardContent.addEventListener("touchmove", function() {
            event.stopPropagation();
            console.log("weatherCardContent touchmove!");
            if (moveOverCheck.check(event)) {
                weatherCardContentMove = true;
            }
            weatherCardContent.style.backgroundColor = "";
        }, false);
        weatherCardContent.addEventListener("touchend", function() {
            event.stopPropagation();
            console.log("weatherCardContent touchend! weatherCardContentMove is " + weatherCardContentMove);
            if (false == weatherCardContentMove) {
                if (weatherCardData.content.bottomButton.url) {
                    exec("UIComponents", 'openBrowser', [ "", weatherCardData.content.bottomButton.url]);
                }
                opCodeKey = "content";
                exec("WidgetMMPContainerComponents", "appenCardCpaLog", ["weather"]);
                exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.weather, opCodeKey]);
            }
            weatherCardContent.style.backgroundColor = "";
        }, false);

		weatherCardContent.refreshContent = function(weatherResult) {
		    console.log("weatherCardContent.refreshContent");
			weatherCardContent.style.visibility = "visible";
			var weatherTempData = weatherResult.low + "°/" + weatherResult.high + "°";
			var cityInfo = weatherResult.city;
            if (cityInfo != null && cityInfo != undefined && cityInfo != "") {
                if (cityInfo.length > 9) {
                    cityInfo = cityInfo.substring(0, 7) + "...";
                }
            }
			var weatherDescriptionData = cityInfo + "  " +  weatherResult.condition.description;
			var weatherBgImg = weatherResult.condition.image;
			weatherTemp.innerText = weatherTempData;
			weatherDescription.innerText = weatherDescriptionData;
            if (weatherResult.pm25 && weatherResult.pm25.num && weatherResult.pm25.desc) {
                weatherPm25.innerText = "空气质量: " + weatherResult.pm25.desc;
            } else {
                weatherPm25.innerText = "空气质量: 无";
            }
			rightImgDiv.style.display = "block";
			weatherDefaultImg.style.display = "block";
            weatherImg.style.display = "none";
            weatherImg.src = "";
            weatherImg.onload = function() {
                console.log("weatherImg onload sucess!");
                weatherDefaultImg.style.display = "none";
                weatherImg.style.display = "block";
            };
			weatherImg.src = weatherBgImg;
		};

		weatherCardContent.appendChild(leftContentDiv);
		weatherCardContent.appendChild(rightImgDiv);
		return weatherCardContent;
	}

	/**
	 * 创建天气提醒引导区域
	 */
	function createWeatherCardScheduleGudie(weatherData) {
		console.log("createWeatherCardScheduleGudie()");
		var weatherScheduleGuide = document.createElement("div");
		weatherScheduleGuide.className = "css_weather_schedule_guide";

        var weatherScheduleGuideDiv = document.createElement("div");
        weatherScheduleGuideDiv.className = "css_weather_schedule_guide_div";
		//天气提醒文字提示
		var weatherScheduleGuideText = document.createElement("div");
		weatherScheduleGuideText.className = "css_weather_schedule_guide_text";
		weatherScheduleGuideText.innerText = "工作日早晨，天气预报叫你起床";

		//开启按钮
		var weatherScheduleGuideButton = document.createElement("div");
		weatherScheduleGuideButton.className = "css_weather_schedule_guide_button";
		weatherScheduleGuideButton.innerText = "开启";

        weatherScheduleGuideDiv.appendChild(weatherScheduleGuideText);
        weatherScheduleGuideDiv.appendChild(weatherScheduleGuideButton);

		//清楚按钮
		var weatherScheduleGuideClear = document.createElement("div");
		weatherScheduleGuideClear.className = "css_weather_schedule_guide_clear";

		weatherScheduleGuideClearImg = document.createElement("img");
        weatherScheduleGuideClearImg.className = "css_weather_schedule_guide_clear_img";
		weatherScheduleGuideClearImg.src = weather_schedule_guide_clear;
		weatherScheduleGuideClear.appendChild(weatherScheduleGuideClearImg);

		weatherScheduleGuide.appendChild(weatherScheduleGuideDiv);
		weatherScheduleGuide.appendChild(weatherScheduleGuideClear);

		if (checkWeatherResult(weatherData)) {
			weatherScheduleGuide.style.display = "block";
		} else {
			weatherScheduleGuide.style.display = "none";
		}

		//添加点击事件
		var weatherScheduleGuideMove = false;
		weatherScheduleGuide.addEventListener("touchstart", function() {
			event.stopPropagation();
			console.log("weatherScheduleGuideMove touchstart! Event target is " + event.target);
			moveOverCheck.start(event);
			weatherScheduleGuideMove = false;
		}, false);
		weatherScheduleGuide.addEventListener("touchmove", function() {
			event.stopPropagation();
			console.log("weatherScheduleGuideMove touchmove! Event target is " + event.target);
			if (moveOverCheck.check(event)) {
				weatherScheduleGuideMove = true;
			}
		}, false);
		weatherScheduleGuide.addEventListener("touchend", function() {
			event.stopPropagation();
			if (false == weatherScheduleGuideMove) {
				//将不显示提醒引导的标志位置为false并缓存起来
				weatherCardData.showScheduleGuide = false;
				cacheMgr.setCache(weatherCardData.id, JSON.stringify(weatherCardData));
				
				//根据用户点击的区域不同，做相应的处理
				console.log("weatherScheduleGuideMove ! Event target is " + event.target);
				if (event.target == weatherScheduleGuideClearImg || event.target == weatherScheduleGuideClear) {
					console.log("click object is weatherScheduleGuideClear");
					weatherScheduleGuide.style.display = "none";
					//记录稽核日志和操作日志
                    opCodeKey = "scheduleGuideClear";
                    exec("WidgetMMPContainerComponents", "appenCardCpaLog", ["schedule"]);
                    exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.weather, opCodeKey]);
				} else if (event.target == weatherScheduleGuideText || event.target == weatherScheduleGuideButton || event.target == weatherScheduleGuideDiv) {
                    console.log("click object is weatherScheduleGuideText or weatherScheduleGuideButton");
                    weatherScheduleGuide.style.display = "none";
                    exec("WidgetMMPContainerComponents", "clickWeatherGuidBtn", []);
                    //记录稽核日志和操作日志
                    opCodeKey = "scheduleGuide";
                    exec("WidgetMMPContainerComponents", "appenCardCpaLog", ["schedule"]);
                    exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.weather, opCodeKey]);
                }
			}
		}, false);

		return weatherScheduleGuide;
	}

	/**
	 *创建天气卡片的bottom区域
	 */
	function createWeatherCardBottom(weatherData) {
		console.log("createWeatherCardBottom()");
		var weatherCardBottom = document.createElement("div");
		weatherCardBottom.className = "css_card_bottom_btn";
		weatherCardBottom.innerText = "查看更多";

		if (weatherData && weatherData.bottomButton) {
			weatherCardBottom.style.visibility = "visible";
			weatherCardBottom.innerText = weatherData.bottomButton.name;
		} else {
			weatherCardBottom.style.visibility = "hidden";
		}

		var weatherCardBottomMove = false;
		weatherCardBottom.addEventListener("touchstart", function() {
			event.stopPropagation();
			console.log("weatherCardBottom touchstart!");
			moveOverCheck.start(event);
			weatherCardBottom.style.backgroundColor = "#efefef";
			weatherCardBottomMove = false;
		}, false);
		weatherCardBottom.addEventListener("touchmove", function() {
			event.stopPropagation();
			console.log("weatherCardBottom touchmove!");
			if (moveOverCheck.check(event)) {
				weatherCardBottomMove = true;
			}
			weatherCardBottom.style.backgroundColor = "";
		}, false);
		weatherCardBottom.addEventListener("touchend", function() {
			event.stopPropagation();
			console.log("weatherCardBottom touchend!");
			if (false == weatherCardBottomMove) {
                if (weatherCardData.content.bottomButton.url) {
                    exec("UIComponents", 'openBrowser', [ "", weatherCardData.content.bottomButton.url]);
                }

				//记录稽核日志和操作日志
				opCodeKey = "moreBtn";
                exec("WidgetMMPContainerComponents", "appenCardCpaLog", ["weather"]);
                exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.weather, opCodeKey]);
			}
			weatherCardBottom.style.backgroundColor = "";
		}, false);
		
		
		weatherCardBottom.refreshBottom = function(weatherResult) {
			if (weatherResult && weatherResult.bottomButton) {
				weatherCardBottom.style.visibility = "visible";
				weatherCardBottom.innerText = weatherResult.bottomButton.name;
				weatherCardBottom.url = weatherResult.bottomButton.url;
			}
		};

		return weatherCardBottom;
	}
	
	function createWeatherCardLoadView() {
	    var loadState = "";
	    
		var weatherCardLoadView = document.createElement("div");
		weatherCardLoadView.className = "css_weather_load_view";
		
		//加载出错提示区域
		var weatherCardLoadFailDiv = document.createElement("div");
		weatherCardLoadFailDiv.className = "css_weather_load_fail_div";
		weatherCardLoadFailDiv.innerText = "加载失败，点击重试";
		
		//加载动画区域
		var weatherCardLoadingDiv = document.createElement("div");
		weatherCardLoadingDiv.className = "css_weaher_loading_div";
		
		var weatherCardLoadingImg = document.createElement("img");
        weatherCardLoadingImg.className = "css_weather_loading_img";
        weatherCardLoadingImg.src = frame_loading_img;
		
		//加载的提示语
		var weatherCardLoadingTip = document.createElement("div");
		weatherCardLoadingTip.className = "css_weather_loading_tip";
		weatherCardLoadingTip.innerText = "正在加载";
		
		weatherCardLoadingDiv.appendChild(weatherCardLoadingImg);
		weatherCardLoadingDiv.appendChild(weatherCardLoadingTip);
		
		weatherCardLoadView.appendChild(weatherCardLoadFailDiv);
		weatherCardLoadView.appendChild(weatherCardLoadingDiv);
		
		var weatherCardLoadViewMove = false;
        weatherCardLoadView.addEventListener("touchstart", function() {
            event.stopPropagation();
            console.log("weatherCardLoadView touchstart!");
            moveOverCheck.start(event);
            weatherCardLoadView.style.backgroundColor = "#efefef";
            weatherCardLoadViewMove = false;
        }, false);
        weatherCardLoadView.addEventListener("touchmove", function() {
            event.stopPropagation();
            console.log("weatherCardLoadView touchmove!");
            if (moveOverCheck.check(event)) {
                weatherCardLoadViewMove = true;
            }
            weatherCardLoadView.style.backgroundColor = "";
        }, false);
        weatherCardLoadView.addEventListener("touchend", function() {
            event.stopPropagation();
            console.log("weatherCardLoadView touchend! loadState is " + loadState);
            if (false == weatherCardLoadViewMove) {
                if (loadFailState.LOAD_FAIL == loadState) {
                    weatherCardLoadView.show(loadFailState.LOADING);
                    if (!isWeatherCardRequesting) {
                    	requestTrigger = 0;
                        startRequestWeather();
                    }
                    //记录操作日志
                    opCodeKey = "loadFailRetry";
                    exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.weather, opCodeKey]);
                } else if (loadFailState.NONE_LOCATION == loadState) {
                    weatherCardLoadView.show(loadFailState.LOADING);
                    exec("WidgetMMPContainerComponents", "requestLocate", []);
                    //记录操作日志
                    opCodeKey = "noneLocationRetry";
                    exec("WidgetMMPContainerComponents", "appenCardOpLog", [cardOpCode.weather, opCodeKey]);
                }
                
                //记录稽核日志和操作日志
                exec("WidgetMMPContainerComponents", "appenCardCpaLog", ["weather"]);
            }
            weatherCardLoadView.style.backgroundColor = "";
        }, false);
		
		function startLoading() {
            //停止加载动画
            if (contentRotateId) {
                stopRotateAnimation(contentRotateId);
            }

            //开始动画
            contentRotateId = startRotateAnimation(weatherCardLoadingImg);
        }

        function stopLoading() {
            //停止加载动画
            if (contentRotateId) {
                stopRotateAnimation(contentRotateId);
                contentRotateId = null;
            }
        }
		
		weatherCardLoadView.show = function(state, tip) {
		    console.log("weatherCardLoadView.show(), state is " + state + ", tip is " + tip);
		    loadState = state;
		    if (loadFailState.LOADING == state) {
		        weatherCardLoadFailDiv.style.display = "none";
		        weatherCardLoadingDiv.style.display = "block";
		        weatherCardLoadView.style.display = "block";
		        startLoading();
		    } else if (loadFailState.LOAD_FAIL == state || loadFailState.NOT_SUPPORT == state || loadFailState.NONE_LOCATION == state) {
		        stopLoading();
		        weatherCardLoadFailDiv.style.display = "block";
                weatherCardLoadingDiv.style.display = "none";
                weatherCardLoadView.style.display = "block";
                if (tip) {
                    weatherCardLoadFailDiv.innerText = tip;
                }
		    } else if (loadFailState.DISAPPEAR == state) {
		        stopLoading();
		        weatherCardLoadFailDiv.style.display = "none";
                weatherCardLoadingDiv.style.display = "none";
                weatherCardLoadView.style.display = "none";
		    } else if (loadFailState.LOCATING == state) {
		        weatherCardLoadFailDiv.style.display = "block";
                weatherCardLoadingDiv.style.display = "none";
                weatherCardLoadView.style.display = "block";
                if (tip) {
                    weatherCardLoadFailDiv.innerText = tip;
                }
		    }
		};

		return weatherCardLoadView;
	}
	
	function createWeatherCardView() {
		console.log("createWeatherCardView()");
		
		var weatherData = null;
		if (weatherCardData) {
			weatherData = weatherCardData.content;
		}
		console.log("weatherData is " + JSON.stringify(weatherData));
        
        var weatherArea = document.createElement("div");
        weatherArea.className = "css_card";
        
		//创建天气卡片的title区域
		//var weatherCardTitle = createWeatherCardTitle(weatherData);
        var weatherCardTitle = createCardMainTitle(weatherCardData.id, weatherCardData.name,weatherCardData.caseList[0],titleResponseListener, weatherCardData.caseRedDot);
		if (weatherCardTitle) {
            weatherArea.appendChild(weatherCardTitle);
        }

		//创建天气卡片的content区域
		var weatherCardContent = createWeatherCardContent(weatherData);
		weatherArea.appendChild(weatherCardContent);

		//创建天气提醒引导区域(需要显示时)
        var weatherCardScheduleGuide = null;
		console.log("weatherCardData.showScheduleGuide is " + weatherCardData.showScheduleGuide);
		if (true == weatherCardData.showScheduleGuide || "true" == weatherCardData.showScheduleGuide) {
			weatherCardScheduleGuide = createWeatherCardScheduleGudie(weatherData);
			weatherArea.appendChild(weatherCardScheduleGuide);
		}
		
		//创建天气卡片的bottom区域
		var weatherCardBottom = createWeatherCardBottom(weatherData);
		weatherArea.appendChild(weatherCardBottom);
		
		//创建天气卡片的加载区域
		var weatherCardLoadView = createWeatherCardLoadView();
		weatherCardLoadView.show(loadFailState.DISAPPEAR);
		weatherArea.appendChild(weatherCardLoadView);
		
		//创建底部的阴影区
        var weatherCardBottomShadow = document.createElement("div");
        weatherCardBottomShadow.className = "css_card_bottom_shadow";
        weatherArea.appendChild(weatherCardBottomShadow);
		
		//更新天气视图
		weatherCardDiv.refreshWeatherView = function(weatherResult, isWeatherDataChanged) {
		    console.log("weatherCardDiv.refreshWeatherView(), weatherRelsult is " + JSON.stringify(weatherResult) + ", isDataChanged is " + isWeatherDataChanged);
			//weatherCardTitle.stopLoading();
			weatherCardLoadView.show(loadFailState.DISAPPEAR);
			weatherCardContent.style.display = "block";
            weatherCardBottom.style.display = "block";
			
			//更新天气视图的title
			/*var updateDate = weatherResult.date;
			if (updateDate) {
				weatherCardTitle.refreshTitle(updateDate);
			}*/

            if (isWeatherDataChanged) {
                //更新天气视图的content
                weatherCardContent.refreshContent(weatherResult);

                //更新天气视图的天气提醒引导
                if (true == weatherCardData.showScheduleGuide || "true" == weatherCardData.showScheduleGuide) {
                    if (weatherCardScheduleGuide) {
                        weatherCardScheduleGuide.style.display = "block";
                    }
                }
                //更新天气视图的bottom
                weatherCardBottom.refreshBottom(weatherResult);
            }
		};
		
		weatherCardDiv.show = function(state, tip) {
		    console.log("weatherCardDiv.show(), state is " + state + ", tip is " + tip);
			if (loadFailState.LOAD_FAIL == state || loadFailState.NOT_SUPPORT == state || loadFailState.NONE_LOCATION == state) {
			    if (!isHasCacheData) {
			        weatherCardContent.style.display = "none";
                    weatherCardBottom.style.display = "none";
                    weatherCardLoadView.style.display = "block";
                    
                    weatherCardLoadView.show(state, tip);
			    } else {
			    	
			    }
			    if (titleRotateId) {
			        weatherCardTitle.stopLoading();
			        weatherCardTitle.refreshTitle(weatherCardData.content.date);
			    }
			} else if (loadFailState.LOCATING == state) {
			    weatherCardContent.style.display = "none";
                weatherCardBottom.style.display = "none";
                weatherCardLoadView.style.display = "block";
                    
                weatherCardLoadView.show(state, tip);
			}
		};

        //保存变量
        weatherCard = weatherArea;

        if (weatherCardDiv) {
            weatherCardDiv.appendChild(weatherArea);
            weatherCardDiv.showContentPage = showContentPage;
        }

		//设置已经创建的标识位为true
		isWeatherCardCreate = true;
	}
	
	/**
     *判断是否位置信息为空且正在定位,若是则显示“正在加载”，否则则请求天气数据 
     */
	var repeateCount = 1;
	function handleWeatherRequest(isNeedjudgeNet) {
		if(isWeatherCardRequesting) {
			console.log("weather card is requesting");
			return;
		}
	    console.log("handleWeatherRequest() repeateCount = " + repeateCount);

        //获取网络状态
        var netWorkResult = true;
        if(!(false ==isNeedjudgeNet)) {
        	var netWorkResultString = exec("SystemComponents", "isNetWorkAvailable", []);
        	netWorkResult = getResultMessage(netWorkResultString);
        }
        
        console.log("netWorkResult is " + netWorkResult);
        if (true == netWorkResult) {
            //获取位置信息
            var addressJson = getResultMessage(exec("LBSComponents", "getLbsInfo", []));
            console.log("addressJson is " + JSON.stringify(addressJson));
            if (addressJson.latitude && addressJson.longitude) {
                //请求天气数据
                startRequestWeather();
            } else {
                var isLbsStartJson = getResultMessage(exec("LBSComponents", "isStartLBS", []));
                console.log("isLbsStartJson is " + JSON.stringify(isLbsStartJson));
                if (true == isLbsStartJson.lbsStart) {
                    if (repeateCount <= 5) {
                        weatherCardDiv.show(loadFailState.LOCATING, loadFailTips.LOCATING);
                        setTimeout(function() {
                            repeateCount += 1;
                            handleWeatherRequest();
                        }, 3000);
                    } else {
                        weatherCardDiv.show(loadFailState.NONE_LOCATION, loadFailTips.NONE_LOCATION);
                        //回调请求错误给框架
                        listener.callBack(weatherCardData.id, CARD_CONTENT_REQUEST_FAILE);
                    }
                } else {
                    weatherCardDiv.show(loadFailState.NONE_LOCATION, loadFailTips.NONE_LOCATION);
                    //回调请求错误给框架
                    listener.callBack(weatherCardData.id, CARD_CONTENT_REQUEST_FAILE);
                }
            }
        } else {
            weatherCardDiv.show(loadFailState.LOAD_FAIL, loadFailTips.LOAD_FAIL);
            //回调请求错误给框架
            listener.callBack(weatherCardData.id, CARD_CONTENT_REQUEST_FAILE);
            //卡片本身触发的需要toast提示
            if(0 == requestTrigger) {
	            exec("UIComponents","showToast",["网络无连接，请检测网络设置",1]);
            }
        }
	}
	
	function onResourceLoadFinish() {
		console.log("onResourceLoadFinish(), weatherCardData is " + JSON.stringify(weatherCardData));
		
		//创建天气卡片视图
		createWeatherCardView();
		
		//请求天气数据
        requestTrigger = 1;
        if (1 == requestCondition) {
            handleWeatherRequest(false);
        } else if (2 == requestCondition) {
            handleWeatherRequest(true);
        } else {
            //没有网络，不需要请求，但是需要回调
            listener.callBack(weatherCardData.id, CARD_CONTENT_REQUEST_FAILE);
            //显示错误界面
            weatherCardDiv.show(loadFailState.LOAD_FAIL, loadFailTips.LOAD_FAIL);
        }
        
	}
	
	function createWeatherCard(initNetState) {
		var weatherCardId = weatherCardData.id;
		
		//读取缓存内容
		var cacheWeatherString = cacheMgr.getCache(weatherCardId);
		//cacheWeatherString = "{\"id\" : \"weather\",\"name\":\"天气\",\"show\" : true,\"entry\" : \"./weather/card.js\",\"version\" : 1,\"showScheduleGuide\":false, \"content\": {\"city\": \"合肥\",\"date\": \"2014-12-24 16:09\",\"condition\": {\"description\": \"小雨转大雨\",\"image\":\"./weather/res/weather_rain_big.png\",\"bgImage\": \"\"},\"low\": \"8\",\"high\": \"18\",\"pm25\": \"70 轻度污染\",\"bottomButton\": [{\"name\": \"查看更多\",\"url\": \"http://60.166.12.157:8087/LXChannels/sub_weather/index.html\"}]}}";
		console.log("cacheWeatherString is " + cacheWeatherString);
		if (!cacheWeatherString) { //缓存为空，则使用默认数据
		    isHasCacheData = false;
			weatherCardData.showScheduleGuide = true;
			cacheMgr.setCache(weatherCardId, JSON.stringify(weatherCardData));
		} else {
			if(typeof cacheWeatherString == "string") {
				try{
					var weatherCacheCardData = JSON.parse(cacheWeatherString);
					//检查缓存的有效性，即比较版本号
					//若版本号相同的话，则缓存有效，直接使用缓存
					//若版本号不相同的话，则缓存无效，这时需取缓存的“show”字段
					if (weatherCacheCardData.version == weatherCardData.version) {
                        weatherCardData.show = weatherCacheCardData.show;
						weatherCardData.caseRedDot = weatherCacheCardData.caseRedDot;
						console.log("weather caseRedDot= " + weatherCardData.caseRedDot);
                        weatherCardData.showScheduleGuide = weatherCacheCardData.showScheduleGuide;
					    if (weatherCacheCardData.content) {
					        isHasCacheData = true;
                            weatherCardData.content = weatherCacheCardData.content;
					    }
					} else {
					    isHasCacheData = false;
						weatherCardData.show = weatherCacheCardData.show; //保留卡片是否需要显示
						weatherCardData.showScheduleGuide = true; //重置天气提醒引导需要显示
						cacheMgr.setCache(weatherCardId, JSON.stringify(weatherCardData));
					}
				}catch(e){
				    isHasCacheData = false;
				    weatherCardData.showScheduleGuide = true;
				    cacheMgr.setCache(weatherCardId, JSON.stringify(weatherCardData));
					console.log("JSON parse exception");
				}
			}
		}
		
		//若卡片可以显示，则加载资源并创建视图
		console.log("weatherCardData is " + JSON.stringify(weatherCardData));
		if (weatherCardData && weatherCardData.show) {
			if (initNetState) {
                requestCondition = 1;
            }
            else {
                requestCondition = 0;
            }
			//加载css资源文件
			xcssParser.parseStyleFile("./weather/business.x.css", onResourceLoadFinish);
		} else {
		    //卡片不需要显示时，需要回调给框架不需要请求
		    listener.callBack(weatherCardData.id, CARD_CONTENT_NO_REQUEST);
		    requestCondition = 2;
		}
	}

	function createWeatherCardFrame(cardDefaultData, initNetState) {
		console.log("createWeatherCardFrame(), cardDefalutData is " + JSON.stringify(cardDefaultData));
		weatherCardData = cardDefaultData;
		
		weatherCardDiv = document.createElement("div");
		weatherCardDiv.id = weatherCardData.id;
		
		createWeatherCard(initNetState);
		
		weatherCardDiv.updateData = function(){
            if (weatherCardData.show) {
                //不需要判断网络直接请求
                if(!isWeatherCardRequesting){
	                requestTrigger = 1;
                }
                startRequestWeather(false);
            } else {
            	//防止长时间定位，导致不回调
                listener.callBack(weatherCardData.id, CARD_CONTENT_NO_REQUEST);
            }
        };

        //更新卡片是否显示的状态
        weatherCardDiv.updateState = function(state) {
            console.log("weatherCardDiv.updateCardState, state is " + state);
            if (true == state || "true" == state) { //显示卡片
                if (isWeatherCardCreate) { //天气卡片视图已创建
                    console.log("isWeatherCardRequested is " + isWeatherCardRequested);
                    weatherCardDiv.style.display = "block";
                    if (!isWeatherCardRequested) {
                        //未成功请求过，不仅需要显示视图，而且还需进行请求
                        if (!isWeatherCardRequesting) {
                        	requestTrigger = 1;
                            startRequestWeather();
                        }
                    }
                } else { //天气卡片视图未创建
                    xcssParser.parseStyleFile("./weather/business.x.css", onResourceLoadFinish);
                }
            } else if (false == state || "false" == state) { //不显示卡片
                weatherCardDiv.style.display = "none";
            }

            //更新缓存
            weatherCardData.show = state;
            cacheMgr.setCache(weatherCardData.id, JSON.stringify(weatherCardData));
        };

        weatherCardDiv.date_type = "card";
        weatherCardDiv.initShowState = weatherCardData.show;
		
		return weatherCardDiv;
	}

    function operateCallBack(callBackId) {
        switch(callBackId) {
            case 0://取消
                if (zdOffDom) {
                    zdOffDom.style.display = "none";
                }
                weatherCard.style.display = "block";
                break;
            case 1://置顶
                if (zdOffDom) {
                    zdOffDom.style.display = "none";
                }
                weatherCard.style.display = "block";
                stickTopCardByDom(weatherCardDiv);
                break;
            case 2://关闭
                //显示关闭确认弹框
                //showCloseConfirmDialogById(weatherCardData.id);
                break;

            default:
                break;
        }
    }
    function speechCaseCallBack(){
        if(zdOffDom) {
            zdOffDom.style.display = "none";
        }
        if(speechCaseDom) {
            speechCaseDom.style.display = "none";
        }
        weatherCard.style.display = "block";
    }
    function titleResponseListener(responseId){
        switch(responseId) {
            case 0://点击说法示例区域
                if(weatherCardData && weatherCardData.caseRedDot) {
                    weatherCardData.caseRedDot = false;
                    cacheMgr.setCache(weatherCardData.id, JSON.stringify(weatherCardData));
					console.log("save red dot state false weather");
                }
                if(zdOffDom) {
                    zdOffDom.style.display = "none";
                }

                weatherCard.style.display = "none";
                if (speechCaseDom) {
                    speechCaseDom.style.display = "block";
                } else {
                    speechCaseDom = createSpeechCaseView(weatherCardData.id, weatherCardData.name, weatherCardData.caseList);
                    if (speechCaseDom) {
                        speechCaseDom.setCancelListener(speechCaseCallBack);
                        weatherCardDiv.appendChild(speechCaseDom);
                    }
                }
                break;
            case 1://点击置顶箭头按钮
                if(speechCaseDom) {
                    speechCaseDom.style.display = "none";
                }
                weatherCard.style.display = "none";

                if (zdOffDom) {
                    zdOffDom.style.display = "block";
                } else {
                    zdOffDom = createOperationView(weatherCardData.id, weatherCardData.name, weatherCardData.id);
                    if (zdOffDom) {
                        zdOffDom.setOpListener(operateCallBack);
                        weatherCardDiv.appendChild(zdOffDom);
                    }
                }
                break;
            case 2://刷新，暂时没有实现
                break;
            default:
                break;
        }
    }
    function showContentPage(){
        if(zdOffDom) {
            zdOffDom.style.display = "none";
        }
        if(speechCaseDom) {
            speechCaseDom.style.display = "none";
        }
        weatherCard.style.display = "block";
    }

    window.startWeatherRequest = function() {
        if (!isWeatherCardRequesting) {
            startRequestWeather();
        }
    };
    window.locateFail = function() {
        weatherCardDiv.show(loadFailState.NONE_LOCATION, loadFailTips.NONE_LOCATION);
    };

	return createWeatherCardFrame;
}); 